#!/usr/bin/ruby

require 'logger'
require 'yaml'
require 'pp'
require 'pathname'

require 'rubygems'
require 'beaneater'
require 'daemons'

$LOAD_PATH.unshift(File.dirname(__FILE__) + '/../lib')

require 'ForemanVM'

# Based on:
# http://www.jstorimer.com/blogs/workingwithcode/7766093-daemon-processes-in-ruby
def daemonize_app
  exit if fork
  Process.setsid
  exit if fork
  STDIN.reopen "/dev/null"
  STDOUT.reopen "/dev/null", "a" 
  STDERR.reopen "/dev/null", "a" 
end

#
# MAIN()
#

if ARGV.include? '--daemon'
  daemonize_app 
  logfile = File.dirname(__FILE__) + '/../log/worker.log'
  log = Logger.new(logfile, 10, 1024000)
  log.info 'started a background worker process'
else 
  log = Logger.new(STDERR)
  log.info 'started a foreground worker process'
end

if ARGV.include? '--debug'
  log.level = Logger::DEBUG
end


# DEADWOOD: should use #clear method instead
### Kill any buried jobs
##if config['reap_buried_jobs']
##  while tube.peek(:buried)
##    log.info tube.stats
##    tube.kick(1)
##    job = tube.reserve 
##    log.info 'discarding job for ' + job.body
##  end
##end

wq = ForemanAP::Workqueue.new

wq.process_all_jobs do |rec|
  begin

  raise 'user is required' unless rec.has_key?('user')
  raise 'action is required' unless rec.has_key?('action')
  raise 'invalid API version' unless rec['api_version'] == 1
  raise 'invalid user' unless rec['user'] =~ /^([a-zA-Z0-9_.]+)$/m
  raise 'invalid action' unless rec['action'] =~ /^([a-zA-Z0-9_.]+)$/m

  hostname = rec['buildspec']['name'] or raise 'name is required'
  domain = rec['buildspec']['domain'] or raise 'domain is required'

  log.info("processing a #{rec['action']} job for hostname #{hostname}: #{rec.inspect}")
  
    fvm = ForemanVM.new
    case rec['action']
    when 'create'
      fvm.name = hostname
      fvm.buildspec = rec['buildspec']
      fvm.create
    when 'rebuild'
      fvm.name = hostname + '.' + domain
      fvm.buildspec = rec['buildspec']
      fvm.rebuild
    when 'destroy'
      fvm.name = hostname + '.' + domain
      fvm.delete
    when 'snapshot-create'
      fvm.snapshot_create
    when 'snapshot-revert'
      fvm.snapshot_revert
    else
      log.error "invalid action: #{action}"
      raise "invalid action: #{action}"
    end
    log.info("job for #{hostname} is complete")
    puts "done"
  rescue => e
    log.error("job for #{hostname} failed")
    log.error e.inspect
    log.error e.backtrace
  end
end
