class Architect
  require 'bundler/setup'
  require 'architect/config'
  require 'architect/designer'
  require 'architect/log'
  require 'architect/plan'
  require 'architect/plugin_manager'
  # FIXME: causes slowdown due to libvirt connection setup
  #require 'architect/report'

  require 'pp'

  attr_accessor :config

  def plugins
    @plugins ||= Architect::PluginManager.new(@config.to_hash[:plugins])
  end

  def initialize(opts = default_options)
    @config = Architect::Config.new(opts[:conffile])
    @plugins = nil   # PluginManager, initialized on-demand
  end

  # Build an instance based on a specification
  def build(spec)
   builder(spec).build(spec)
  end

  # Return the appriate Builder object for an instance
  def builder(spec)
    case spec[:instance_type]
    when 'virtual'
      require 'architect/builder/virtual'  # Slow, so load it on demand
      @vm_builder ||= VirtualMachineBuilder.new
    when 'physical'
      require 'architect/builder/physical'
      @physical_builder ||= PhysicalMachineBuilder.new
    else
      pp spec
      raise 'Unsupported instance type'
    end
  end

  def default_options
   return({
	conffile: nil,
    })
  end
end
