class Architect
  # Parse the configuration file and ARGV variables
  class Config
    require 'facter'
    require 'pp'

    def to_hash
      @config
    end

    def initialize(conffile = nil)
      config = {
        :domain => Facter['domain'].value,
        :plugins => {},
      }
      if conffile.nil? or conffile.kind_of?(String)
        conffile ||= File.dirname(__FILE__) + "/../../conf/architect.yaml"
        conffile = File.realpath(conffile)
        if File.exist?(conffile)
          config.merge! parse(conffile)
        else
          raise "configuration file #{conffile} not found"
        end
      elsif conffile.kind_of?(Hash)
        config.merge! symbolize(conffile)
      else
        raise ArgumentError
      end
      config.keys.each { |k| publish k }
      @config = config
    end

    # TODO: find a better way to do this
    def self.symbolize_hash(obj)
      symbolize(obj)
    end

    private

    # Given a nested hash, convert all keys from String to Symbol type
    # Based on http://stackoverflow.com/questions/800122/best-way-to-convert-strings-to-symbols-in-hash
    #
    def symbolize(obj)
      return obj.inject({}){|memo,(k,v)| memo[k.to_sym] =  symbolize(v); memo} if obj.is_a? Hash
      return obj.inject([]){|memo,v    | memo           << symbolize(v); memo} if obj.is_a? Array
      return obj
    end

    # Parse a configuration file and return a Hash
    def parse(path)
      # FIXME: want to disallow group-readable also
      raise "Insecure permissions on #{path}; please chmod to 0600" \
	if File.stat(path).world_readable? 
      symbolize(YAML.load_file(path))
    end

    # Given a key, create a read-only accessor method
    def publish(key)
      self.class.class_eval do
        define_method(key.to_sym) { @config[key.to_sym] }
      end
    end
  end
end
