class Architect
  # When the main Architect executable runs, it calls out to various
  # Plugin objects to do additional work at various stages.
  class Plugin
    require 'architect/log'

    # The name of the plugin
    attr_reader :name
    # The configuration settings for the plugin
    attr_reader :config

    # Called when the plugin is registered
    def register
      nil
    end

    # Parse a [+yaml+] configuration file
    def configure(yaml)
      nil
    end

    # Check to see if any actions need to be taken, and return
    # a list of Architect::ChangeRequest objects for each proposed
    # action
    def check
      []
    end

    # Ask questions during the design of a new plan; i.e. when
    # architect --design is called.
    def design
      nil
    end

    # Parse a YAML plan file and perform validation.
    def plan(yaml)
      nil
    end

    # Execute the proposed actions based on the current plan.
    def execute
      nil
    end

    # A single instance of a machine (physical, virtual, or container)
    class MachineInstance
      def initialize(spec)
        raise 'Method not implemented'
      end

      # Destroy the instance
      def destroy
        raise 'Method not implemented'
      end

      # Change the name of an instance to [+newname+]
      def rename(newname)
        raise 'Method not implemented'
      end

      # Return true if the instance exists within the realm of whatever the Plugin manages.
      def exist?
        raise 'Method not implemented'
      end
    end
  end
end
