class Architect
  class PluginManager
    require 'ostruct'
    require 'architect/plugin'

    # The path to the configuration directory for plugins.
    # Each plugin gets it's own configuration file with the same
    # name as the plugin.
    attr_accessor :pluginconfigdir

    attr_reader :plugins

    def initialize(config)
      @config = config.to_hash
      @pluginconfigdir = '/etc/architect/plugin.d/'
      @log = Architect::Log.log
      #log.level = Logger::DEBUG
      @plugins = OpenStruct.new
      plugindir =  File.dirname(__FILE__) + '/plugin'
      log.debug "registering plugins in #{plugindir}"
      Dir.glob("#{plugindir}/*.rb").each do |pluginfile|
        load pluginfile
      end
      register_all_plugins
      plugins.each do |plugin|
        conffile = "#{pluginconfigdir}/#{plugin.name}.yaml"
        if File.exist? conffile
          plugin.configure Architect::Config.symbolize_hash YAML.load(conffile)
        end
      end
    end

    # Before the program exist, each plugin is unregistered.
    # This gives it a chance to clean up.
    def unregister_all
      raise 'FIXME - STUB'
    end

    private

    attr_accessor :log

    def load(pluginfile)
      log.debug "loading #{pluginfile}"
      begin
        require pluginfile
      rescue
        log.warn "plugin failed to load: #{pluginfile}"
      end
    end

    # Instantiate all plugins
    def register_all_plugins
      ObjectSpace.each_object(Class) do |klass|
        if klass < Architect::Plugin
          plugin = klass.new
          name = plugin.name.to_sym
          @plugins[name] = plugin
          log.debug "registered #{klass} as #{name}"
        end
      end
    end
  end
end
