module ForemanAP
  # A build specification 
  class BuildSpec 

    # Short hostname
    attr_accessor :name
    # DNS domain name
    attr_accessor :domain
    # Number of CPUs
    attr_accessor :cpus
    # Amount of memory
    attr_accessor :memory
    # Amount of disk space. Multiple disks can be separated with a comma.
    attr_accessor :disk_capacity
    # The disk format; either raw or qcow2
    attr_accessor :disk_format
    # The libvirt storage pool
    attr_accessor :storage_pool
    # Network interface
    attr_accessor :network_interface

    # Generate output suitable for feeding into the Foreman API
    def to_foreman_api
      rec = {}
      rec['compute_attributes'] = {}
      rec['compute_attributes']['volumes_attributes'] = disk_capacity_to_api
      rec
    end

  private

  # Get the Foreman API equivalent for disk capacity
  def disk_capacity_to_api
    res = {}
    disks = @disk_capacity
    volume_id = 0
    disks.split(',').each do |disk_size|
      res[volume_id.to_s] = {
        'capacity' => disk_size,
        'pool_name' => @storage_pool,
        'format_type' => @disk_format,
      }
      volume_id += 1
    end
    res
  end
  end
end
