module ForemanAP
  # Parse the configuration file and provide configuration variables.
  #
  class Config
  
    require 'yaml'
    
    # The user account to login to Foreman as.
    attr_reader :foreman_user
    # The password to login to Foreman with.
    attr_reader :foreman_password
    # The URI of the Foreman server.
    attr_reader :foreman_uri
    # The user account to login to libvirtd as.
    attr_reader :libvirt_user
    # The password to login to libvirtd with.
    attr_reader :libvirt_password
    # DEPRECATED -- do not use
    attr_reader :foreground
    # DEPRECATED -- do not use
    attr_reader :reap_buried_jobs
    # A list of all hypervisors in the cluster.
    attr_reader :hypervisors
    # The name of the shared storage pool to use on all hypervisors.
    attr_reader :storage_pool
    # The FQDN of the GlusterFS server
    attr_reader :glusterfs_server
    # The email address of the support team to contact if something goes wrong.
    attr_reader :support_contact_email
    
    # Create an object
    # [+conffile+] the path to the configuration file.
    def initialize(conffile = nil)
      if conffile.nil?
        confdir = File.dirname(__FILE__) + '/../../conf'
        conffile = confdir + '/worker.yaml'
      end
      config = {
        :foreground => true,
        :reap_buried_jobs => true,
      }
      case conffile.kind_of?
      when String
        config.merge!(YAML.load_file(conffile))
      when Hash
        config.merge! conffile
      else
        raise ArgumentError
      end

      config.each { |k,v| instance_variable_set("@#{k}", v) }
      config
    end
  end
end
