module ForemanAP

  # Functions related viewing to the virtual machine console
  # 
  class ConsoleViewer
  
    require 'cgi'
    require 'pty'
    require 'timeout'
    
    # If true, output will be formatted for HTML display.
    attr_accessor :html
    
    # Create an object.
    # [+cluster+] a ForemanAP::Cluster object.
    #
    def initialize(cluster)
      @cluster = cluster
      @html = false
      @autoclose = nil
    end

    # Specify a pattern that will cause the console to be automatically
    # closed when it is found in the output.
    #
    #   Example:  / login:/
    #   
    def autoclose=(pattern)
      raise 'Regular expression expected' unless pattern.class == Regexp
      @autoclose = pattern
    end

    # Attach to the serial console of the virtual machine.
    #
    def attach(guest)
      host = @cluster.find(guest)
      puts "Connecting to the serial console of #{guest} via #{host}... "
      print '<pre>' if @html
      ENV['LIBVIRT_AUTH_FILE'] = File.dirname(__FILE__) + '/../../conf/auth.conf'
    begin
      PTY.spawn("virsh -c qemu+tcp://#{host}/system console #{guest}") do |stdin, stdout, pid|
      begin
        # Regularly try to flush the output in a different thread
        # This allows us to detect when the client hangs up even if
        # the main thread is blocked trying to read from the VM console.
        if @html
          t = Thread.new {
            while true
              $stdout.flush or raise 'client has disconnected'
              sleep(5)
            end
          }
          t.abort_on_exception = true
        end
 
        stdout.write ""
        stdout.flush

        stdin.each do |line| 
          if @html
            # TODO: translate ANSI colors into HTML colors
            print CGI::escapeHTML(line).chomp
          else
            print line
          end
          $stdout.flush or exit(0)
          if @autoclose and line =~ @autoclose
            puts "(the console was automatically closed)"
            exit 0
          end
        end
        rescue Errno::EIO
          puts "Errno:EIO error, but this probably just means " +
                "that the process has finished giving output"
        end
      end
      rescue PTY::ChildExited
        puts "The child process exited!"
      end
      print '</pre>' if @html
    end
  end  
end
