class ForemanVM
  require 'optparse'

  # Parse command line options
  #
  def parse_options
    optparse = OptionParser.new do |opts|
      
      opts.on( '--user USER', 'The username to login to Foreman with') do |arg|
        @user = arg
      end
      
      opts.on( '--password PASSWORD', 'The password to login to Foreman with') do |arg|
        @password = arg
      end
      
      opts.on( '--password-file FILE', 'The file containing your Foreman password') do |arg|
        raise 'File does not exist' unless File.exists? arg
        @password = `cat #{arg}`.chomp
      end
      
      opts.on( '--foreman-uri URI', 'The URI of the Foreman password') do |arg|
        @foreman_uri = arg
        #options['foreman_uri'] = arg
      end
      
      
      opts.separator ""
      opts.separator "Specific options for building hosts:"
      
      opts.on( '--console', 'Attach to the virtual machine console') do |arg|
        @buildspec['console'] = true
        @action = 'console' if @action.nil? #XXX-KLUDGE for testing
      end
      
      opts.on( '--cpus CPUS', 'The number of CPUs to assign to the VM') do |arg|
        @buildspec['cpus'] = arg
      end
      
      opts.on( '--memory BYTES', 'The amount of memory, in bytes, to assign to the VM') do |arg|
        @buildspec['memory'] = arg
      end
      
      opts.on( '--domain DOMAIN', 'The DNS domain name to use') do |arg|
        @buildspec['domain'] = arg
      end
      
      opts.on( '--organization ORG', 'The organization name in Foreman') do |arg|
        @buildspec['organization'] = arg
      end

      opts.on( '--owner OWNER', 'The owner name in Foreman') do |arg|
        @buildspec['owner'] = arg
      end
      
      opts.on( '--hostgroup HOSTGROUP', 'The Foreman hostgroup') do |arg|
        @buildspec['hostgroup'] = arg
      end
      
      opts.on( '--compute-resource RESOURCE', 'The Foreman compute resource') do |arg|
        @buildspec['compute_resource'] = arg
      end
      
      opts.on( '--provision-method METHOD', 'The provisioning method (image or network)') do |arg|
        @buildspec['provision_method'] = arg
      end
      
      opts.on( '--environment ENVIRONMENT', 'The Puppet environment') do |arg|
        @buildspec['environment'] = arg
      end
      
      opts.on( '--network-interface INTERFACE', 'The network interface') do |arg|
        @buildspec['network_interface'] = arg
      end
      
      opts.on( '--disk-capacity SIZE', 'The size of the first disk, using M or G suffixes') do |arg|
        @buildspec['disk_capacity'] = arg
      end
      
      opts.on( '--disk-format FORMAT', 'The format of the disk. Can be raw or qcow2') do |arg|
        @buildspec['disk_format'] = arg
      end
      
      opts.on( '--storage-pool POOL', 'The storage pool') do |arg|
        @buildspec['storage_pool'] = arg
      end
      
      opts.on( '--subnet SUBNET', 'The subnet') do |arg|
        @buildspec['subnet'] = arg
      end
      
      # TODO: some nice header thing
      
      opts.separator ""
      opts.separator "Actions:"
      
      opts.on( '--rebuild', 'Rebuild the host') do |arg|
        @action = 'rebuild'
      end
      
      opts.on( '--create', 'Create the host') do |arg|
        @action = 'create'
      end

      opts.on( '--create-storage', 'Create storage and attach it to the host') do |arg|
        @action = 'create-storage'
      end
      
      
      opts.on( '--delete', 'Delete the host') do |arg|
        @action = 'delete'
      end

      opts.on( '--start', 'Power on the VM') do |arg|
        @action = 'start'
      end

      opts.on( '--stop', 'Power off the VM, without doing a graceful shutdown') do |arg|
        @action = 'stop'
      end

      opts.on( '--dumpxml', 'Show the libvirt XML virtual machine definition') do |arg|
        @action = 'dumpxml'
      end

      opts.on( '--monitor-boot', 'Watch the console of a VM as it boots') do |arg|
        @action = 'monitor-boot'
      end

      opts.on( '--enable-libgfapi', 'Enable GlusterFS libgfapi') do |arg|
        @action = 'enable-libgfapi'
      end

      opts.on( '--disable-libgfapi', 'Disable GlusterFS libgfapi') do |arg|
        @action = 'disable-libgfapi'
      end
      
      ## Verbosity (DEBUG, WARN, INFO, etc.)
      ##log_level="DEBUG"
      
      opts.on( '-h', '--help', 'Display this screen' ) do
        puts opts
        exit
      end
    end
    
    optparse.parse!

    ask_password if @password.nil?
  end
end
