module ForemanAP
  # A host in the cluster that runs a hypervisor.
  class Hypervisor
    require 'foreman_vm/storage_pool'

    # The total number of CPUs (physical and virtual)
    def cpus
      nodeinfo.cpus
    end

    # The total amount of memory, in bytes.
    def memory
      nodeinfo.memory * 1024		# Convert from KiB
    end

    # The amount of free memory, in bytes.
    def free_memory
      @conn.node_free_memory
    end
  
    # Lookup a storage pool by name.
    def storage_pool(name)
      StoragePool.new(@conn, name)
    end
  
    # Return the name of hypervisor
    def hostname
      @conn.hostname	
    end

    # A guest running on the hypervisor
    #   [+fqdn+] the FQDN of the guest
    def guest(fqdn)
       ForemanAP::Domain.new(@conn, fqdn, @foreman_api)
    end
      
    # The names of all the domains defined on the hypervisor.
    def domains
      res = []
    
      # Active domains
      @conn.list_domains.each do |domid|
        dom = @conn.lookup_domain_by_id(domid)
        res.push dom.name
      end
    
      # Inactive domains
      @conn.list_defined_domains.each do |domname|
        res.push domname
      end
    
      res
    end

    # A list of handles to all the active domains on the hypervisor
    # TODO - replace #domains with this
    def domains2
      res = []

      # Active domains
      @conn.list_domains.each do |domid|
        dom = @conn.lookup_domain_by_id(domid)
        res.push ForemanAP::Domain.new(@conn, dom.name, @foreman_api)
      end

      res
    end
    
    # Create an object.
    #
    # [+uri+] The libvirt URI. Example: qemu+tcp:///host/system
    # [+user+] The username to login with.
    # [+passphrase+] The passphrase to login with.
    def initialize(uri, user, passphrase)
      @uri = uri
      @conn = Libvirt::open_auth(@uri, 
    		[Libvirt::CRED_AUTHNAME, Libvirt::CRED_PASSPHRASE]) do |cred|
         case cred["type"]
         when Libvirt::CRED_AUTHNAME 
           user
         when Libvirt::CRED_PASSPHRASE
           passphrase
         else
           raise 'Unknown error'
         end
      end
    end

    private
    
    # The 'nodeinfo' structure for the connection
    def nodeinfo
      @nodeinfo ||= @conn.node_get_info
    end
  end
end
