module ForemanAP
  # A storage pool containing disk volumes.
  class StoragePool
    require 'foreman_vm/volume'
  
    # Scan the pool for changes. This is useful for pools that are shared
    # across multiple hosts in a cluster.
    def refresh
      @sph.refresh
      true
    end
  
    # Lookup a volume by +name+, and return an object handle.
    def volume(name)
      Volume.new(@sph, name)
    end
  
    # Create a new object.
    #
    # [+conn+] A libvirt connection handle.
    # [+name+] The name of the storage pool.
    #
    def initialize(conn, name)
      @sph = conn.lookup_storage_pool_by_name(name)
    end

    # Create a new volume within the storage pool.
    # [+path+] the full path to the volume
    # [+capacity+] the disk capacity, in bytes
    #
    def create_volume(path,capacity)
      xml = "<volume>
               <name>#{File.basename(path)}</name>
               <key>#{path}</key>
               <source>
               </source>
               <capacity unit='bytes'>#{capacity}</capacity>
               <allocation unit='bytes'>197120</allocation>
               <target>
                 <path>#{path}</path>
                 <format type='raw'/>
                 <permissions>
                   <mode>0660</mode>
                   <owner>107</owner>
                   <group>107</group>
                 </permissions>
               </target>
             </volume>
      "
      #TODO: @log.debug "creating volume: #{xml}"
      @sph.create_volume_xml(xml)
     end
  end
end

#--
## Legacy code below here

class ForemanVM
 
# Copy an existing disk volume to reduce the build time
# 
def copy_volume
  delete_volume

  virsh "vol-clone --pool #{@buildspec['storage_pool']} --vol #{@buildspec['_disk_backing_file']} --newname #{fqdn()}-disk1"
end

# Clone an existing disk volume to reduce the build time
# 
def clone_volume
  delete_volume

  # BUG: We would like to do this, but it creates the file owned by root:root
  #virsh "vol-create-as --pool #{@buildspec['storage_pool']} --name #{fqdn()}-disk1 --capacity 30G --format qcow2 --backing-vol #{@buildspec['_disk_backing_file']} --backing-vol-format qcow2"
  #
  # WORKAROUND: use an XML volume definition to set the owner/group
  #  
  xml = "<volume>
  <name>#{fqdn}-disk1</name>
  <key>/gvol/images/#{fqdn}-disk1</key>
  <source>
  </source>
  <capacity unit='bytes'>32212254720</capacity>
  <allocation unit='bytes'>197120</allocation>
  <target>
    <path>/gvol/images/#{fqdn}-disk1</path>
    <format type='qcow2'/>
    <permissions>
      <mode>0660</mode>
      <owner>107</owner>
      <group>107</group>
    </permissions>
  </target>
  <backingStore>
    <path>#{@buildspec['_disk_backing_file']}</path>
    <format type='qcow2'/>
  </backingStore>
</volume>
"
  @log.debug "creating volume: #{xml}"
  virsh("vol-create --pool gvol --file /dev/stdin >/dev/null", xml)
end
end
