module ForemanAP
  # A disk volume attached to a virtual machine.
  class Volume
    # The format of the volume. Currently, only :raw is supported.
    def format
      case @vol.info.type
      when 0
        :raw
      else
        raise 'unknown volume type: ' + @sph.info.type
      end
    end
  
    # Delete the volume.
    def delete
      @vol.delete
      true
    end
  
    def initialize(pool, name)
      @pool = pool
      @name = name
      @vol = pool.lookup_volume_by_name(name)
    end
  end
end

#--
## Legacy code below here

class ForemanVM
# Delete the disk volume associated with the VM
# 
def delete_volume
  virsh "vol-delete #{self.fqdn}-disk1 --pool #{@buildspec['storage_pool']}"
end

# Wipe an existing disk volume to fix the permissions
# This is sadly needed to get the uid:gid to be qemu:qemu
# 
def wipe_volume
  delete_volume

  # BUG: We would like to do this, but it creates the file owned by root:root
  #virsh "vol-create-as --pool #{@buildspec['storage_pool']} --name #{fqdn()}-disk1 --capacity 30G --format qcow2 --backing-vol #{@buildspec['_disk_backing_file']} --backing-vol-format qcow2"
  #
  # WORKAROUND: use an XML volume definition to set the owner/group
  #  
  xml = "<volume>
  <name>#{fqdn}-disk1</name>
  <key>/gvol/images/#{fqdn}-disk1</key>
  <source>
  </source>
  <capacity unit='bytes'>32212254720</capacity>
  <allocation unit='bytes'>197120</allocation>
  <target>
    <path>/gvol/images/#{fqdn}-disk1</path>
    <format type='raw'/>
    <permissions>
      <mode>0660</mode>
      <owner>107</owner>
      <group>107</group>
    </permissions>
  </target>
</volume>
"
  @log.debug "creating volume: #{xml}"
  virsh("vol-create --pool gvol --file /dev/stdin >/dev/null", xml)
end
end
