module ForemanAP

  # Allow jobs to be placed on a workqueue and processed later.
  class Workqueue
  
    require 'rubygems'
    require 'beaneater'
    require 'json'
    require 'pp'
  
    def initialize(tube_name = 'foreman-vm')
      @tube_name = tube_name
      @beanstalk = Beaneater::Pool.new(['localhost:11300'])
      @tube = @beanstalk.tubes[tube_name]
    end
  
    # Add an item to the queue
    def enqueue(item)
      @tube.put JSON.pretty_generate(item)
    end
  
    # Remove a job from the queue, and return the body
    def dequeue
      job = @tube.reserve
      result = JSON.parse(job.body)
      job.delete
      result
    end
  
    # Remove all jobs from the queue
    def clear
      @tube.clear
      true
    end
  
    # Return a list of all jobs
    #   (TODO: try to avoid leaking beanstalkd details)
    def jobs
      buf = "job stats:\n\n"
      buf += @tube.stats.inspect + "\n\n\n" 
      buf
    end

    # Process all jobs
    def process_all_jobs
      @beanstalk.jobs.register(@tube_name) do |job|
        yield JSON.parse(job.body)
      end
  
      @beanstalk.jobs.process!
    end

    # Check if there is a worker waiting for jobs
    def worker?
      @tube.stats.current_watching > 0
    end
  end
end
